from modele_cesar import *

rep_trav = './'

fic_rsv4 = rep_trav + 'dupuit_M1.rsv4'
 
exe = EXEC( RSV4 = fic_rsv4 )
        
#####################################################################################################

# parametres geometriques 
L = 50.                # largeur du domaine etudie
H = 10.                # epaisseur du domaine

# longueur des elements
d = 1. 


# nombre d elements dans la largeur, dans la hauteur et total
nh = int(L/d)
nv = int(H/d)
nelt = nh*nv
# nombre de noeuds sur les lignes horizontales / verticales
nph = 2*nh+1
npv = 2*nv+1
# largeur et hauteur des éléments
disth = L/nh
distv = H/nv

# constitution des coordonnees des noeuds
# on fabrique des lignes horizontales qui comportent 2*nh+1 noeuds
# ou nh+1 noeuds (pour les noeuds aux milieux des cotes verticaux)
x = 0.
y = 0.
nnt = 0

# vcorg represente la liste des coordonnees
vcorg = []

for i in range(nph):
    vcorg.append(x) 
    vcorg.append(y) 
    nnt += 1
    x += disth/2.

for j in range(nv+1):
    x = 0.
    y += distv/2.
    for i in range(nh+1):
        vcorg.extend([x,y]) 
        nnt += 1
        x += disth
    y += distv/2.
    x = 0.
    for i in range(nph):
        vcorg.extend([x,y]) 
        nnt += 1
        x += disth/2.

# definition de l objet coor de la classe COOR


coor = COOR( M = 2, M1 = 0, NNT = nnt, NDIM = 2, VCORG = vcorg )


# constitution de la connectivite

kpnte = [1]
knte = []

ielt = 1
for i in range(nelt):
   ielt += 8
   kpnte.append(ielt)

idecv = 0

i1 = 1 ; i2 = 3 ;     i3 = nph+nh+4 ; i4 = nph+nh+2  
i5 = 2 ; i6 = nph+2 ; i7 = nph+nh+3 ; i8 = nph+1  

for i in range(nv):
    idec1 = 0
    idec2 = 0
    for j in range (nh) :
        n1 = i1+idec1+idecv ; n2 = i2+idec1+idecv  
        n3 = i3+idec1+idecv ; n4 = i4+idec1+idecv
        n5 = i5+idec1+idecv ; n6 = i6+idec2+idecv 
        n7 = i7+idec1+idecv ; n8 = i8+idec2+idecv
        idec1 = idec1+2
        idec2 = idec2+1
        knte.extend( [ n1,n2,n3,n4,n5,n6,n7,n8 ] )
    idecv += nph+nh+1
    
    
# modele avec courbes preprogrammees
modele1 = [DB(NOMG = 'G2',
              ACTI = 'A',
              IMOD = 4,
              INAT = 1,
              CARA = DB_CP( AKXS = 1., AKYS = 1., AKXYS = 0., CE = 1., 
                            A = 1., B = 2., C = 1., D =2., 
                            CER = 0., CES = 0., P0 = 0.1)
              )]

elem = ELEM( M = 0, M1 = 0, NELT = nelt , NGRPE = 1, 
             PNUMEL = kpnte, NUMEL = knte, 
             TYPE = ['DBQ8'] * nelt, 
             GROUPE = [1] * nelt, 
             GRPES = modele1)

# definition des conditions aux limites : 
# on constitue la liste des noeuds situés en x=0 et en x=L
# (de bas en haut)

nd_charge_nulle = []
nd_charge_impos = []
uimp = []
n1g = 1
n1d = 2*nh+1
nd_charge_nulle.append(n1g)
nd_charge_impos.append(n1d)
uimp.append(10.)
for i in range (nv) :
    n1g += 2*nh+1
    n1d += nh+1
    nd_charge_nulle.append(n1g)
    nd_charge_impos.append(n1d)
    uimp.append(10.)
    n1g += nh+1
    n1d += 2*nh+1
    nd_charge_nulle.append(n1g)
    nd_charge_impos.append(n1d)
    uimp.append(10.)

# definition des conditions aux limites

cond = COND( M = 2,
             OPT = [COND_NUL(GEN = [GEN_NUL(IGEN = 2,
                                    NP = len(nd_charge_nulle),
                                    NUM = nd_charge_nulle,
                                    IDL = [1],
                                    NDLN = 1) ] ),
                    COND_IMP(GEN = [GEN_IMP(IGEN = 2,
                                    NP = len(nd_charge_impos),
                                    NUM = nd_charge_impos,
                                    IL = 1,
                                    UIMP = uimp )])
                    ])       

# definition des parametres du calcul

kvcond = KVCOND( KCOND = [1], VCOND = [1.] )

ini = INI( MOD = 'NSAT', IPERM = 1, M2 = 1, VU0 = 0. )

lim = LIM('NSAT', TAB =  kvcond )

sre = SRE( MOD = 'NSAT', KSRE= [1], ISRC =2)

nsat = NSAT( M = 1, V = [0., 1., 0.], 
             MET1 = 'N', MET2 ='H', 
             NITER = 50, TOL = 1.e-3, NPAS1 = 1, 
             T0 = 0. , DT = [ ] , 
             OPT = [ ini, lim, sre ] ) 
              
# constitution du jeu de donnees et lancement
   
jeu = JDD(EXEC = exe,
          COOR = coor,
          ELEM = elem,
          COND = cond,
          NSAT = nsat)
          
jeu.lancer()   

rsv4 = RSV4(jeu, fic_rsv4)

# pour la charge hydraulique, H est la premiere (et la seule)
# composante du dataset
index_compH = 0
H_num = []
H_Dupuit = []
AbscisseH = []
xH = 0.

# pour la vitesse horizontale, Vx est 1re composante du dataset
index_compV = 0
Vx_num = []
Vx_Dupuit = []
AbscisseVx = []
xV = disth/2.
for entity in rsv4.entities:
    if isinstance(entity, TimeStep):
        for table in entity.tables:
            if isinstance(table,NodesByGroup) and table.type_dataset == "Hydraulic Head":
                node = 0
                for ind in range(nph):
                    node += 1
                    valeur_num = table.val[node][index_compH]
                    H_num.append(valeur_num)
                    valeur_analytique = H*(xH/L)**0.5
                    H_Dupuit.append(valeur_analytique)
                    AbscisseH.append(xH)
                    xH += disth/2.
            if isinstance(table,ElementsByGroup) and table.type_dataset == "2D Hydraulic Head velocities":
                elt=0
                for ind in range(nh):
                    elt += 1
                    node = 2
                    valeur_num=table.val[elt][node-1][index_compV]
                    Vx_num.append(valeur_num)
                    valeur_analytique=-H/(2*(xV*L)**0.5)
                    Vx_Dupuit.append(valeur_analytique)
                    AbscisseVx.append(xV)
                    xV += disth

# trace de courbes avec matplotlib 

import matplotlib.pyplot as plt

plt.title('Charge hydraulique sur la base du maillage')
plt.plot(AbscisseH, H_Dupuit)
plt.plot(AbscisseH, H_num)
plt.xlabel("Position (m)")
plt.ylabel("Charge hydraulique (m)")
plt.legend(['Dupuit     ','CESAR     '], loc='lower right')
plt.show()
plt.title('Vitesse horizontale sur la base du maillage')
plt.plot(AbscisseVx, Vx_Dupuit)
plt.plot(AbscisseVx, Vx_num)
plt.xlabel("Position (m)")
plt.ylabel("Vx (m/s)")
plt.legend(['Dupuit     ','CESAR     '], loc='lower right')
plt.show()
